/**********************************************************
 * @brief Simulation code for Binary Consensus in Anonymous Asynchronous Systems using Coins and Failure Detectors.
 * @date July 2018
 * @author Javier Martín Rueda, javier.martin@upm.es
 * @author José Luis López-Presa, joseluis.llopezp@upm.es
 * @author Ernesto Jiménez, ernes@etsisi.upm.es
 **********************************************************/


#ifndef __ARGUMENTS_H__
#define __ARGUMENTS_H__

#include <string>

struct ProgramArguments {
    ProgramArguments() : nMin(0), nMax(0), numXPoints(10), pfMin(0), pfMax(50), pfStep(10), numSimulations(100), failureType(1), debugLevel(0), showProgress(true) {}
    unsigned int nMin; /**< NumProcessesMin */
    unsigned int nMax; /**< NumProcessesMax */
    unsigned int numXPoints; /**< NumXPoints */
    unsigned int pfMin;
    unsigned int pfMax;
    unsigned int pfStep;
    unsigned int numSimulations;
    static const unsigned int FAILURE_NO = 0;
    static const unsigned int FAILURE_START = 1;
    static const unsigned int FAILURE_RANDOM = 2;
    unsigned int failureType;
    unsigned int debugLevel;
    bool showProgress;
    std::string outputFilename;
    std::string comments;
    std::string plotSubtitle;
};

extern ProgramArguments programArguments;
extern std::string failureTypeName[];

void readArguments(int argc, char** argv, ProgramArguments& programArguments);

#endif
