/**********************************************************
 * @brief Simulation code for Binary Consensus in Anonymous Asynchronous Systems using Coins and Failure Detectors.
 * @date July 2018
 * @author Javier Martín Rueda, javier.martin@upm.es
 * @author José Luis López-Presa, joseluis.llopezp@upm.es
 * @author Ernesto Jiménez, ernes@etsisi.upm.es
 **********************************************************/


#ifndef _DRCONSENSUS_H_
#define _DRCONSENSUS_H_

#include "RandomConsensus.h"
#include "FailureDetector.h"
#include "Channel.h"

struct DRConsensusStatistics {
    static const int MAX_R = 100;
    static const int MAX_V = 2;
    DRConsensusStatistics(unsigned int n);
    DRConsensusStatistics(unsigned int n, unsigned int f);
    unsigned int numExecutions, numCorrectExecutions;
    unsigned int n, f;
    ObservedVariable<unsigned int> numRounds;
    unsigned proposed[MAX_V]; /**< Counter per proposed values. */
    unsigned decided[MAX_V]; /**< Counter per decided values. */
    DRConsensusStatistics& operator += (const DRConsensusStatistics& other);
    friend std::ostream& operator << (std::ostream& out, const DRConsensusStatistics& statistics);
    void outputXml(bool xml) { this->xml = xml; };
private:
    bool xml;
};


class DRConsensus : public RandomConsensus {
public:
	DRConsensus(unsigned int numProcesses, int idProcess, Channel& channel, CommonCoin& coin, SyntheticFailureDetector& failureDetector, unsigned int failureType);
	virtual ~DRConsensus();
	virtual int propose(int v);
	DRConsensusStatistics statistics;
private:
    class FilterPhase1;
	SyntheticFailureDetector& failureDetector;
    bool ld;
	int task1();
    MessageList* receivePhase1();
};


class DRConsensus::FilterPhase1 : public MessageFilter {
public:
    FilterPhase1(unsigned int r);
    bool operator()(std::shared_ptr<const Message> mensaje);
private:
    const unsigned int r;
};



#endif
