/**********************************************************
 * @brief Simulation code for Binary Consensus in Anonymous Asynchronous Systems using Coins and Failure Detectors.
 * @date July 2018
 * @author Javier Martín Rueda, javier.martin@upm.es
 * @author José Luis López-Presa, joseluis.llopezp@upm.es
 * @author Ernesto Jiménez, ernes@etsisi.upm.es
 **********************************************************/


#include "Events.h"

#include <sstream>
#include <assert.h>


Event::Event()
{
}


Event::~Event()
{
}


void Event::timestamp(const VectorClock& t)
{
    _t = t;
}

bool VectorClock::operator == (const VectorClock& other) const
{
    if (_t.size() != other._t.size())
        return false;
    for (unsigned int i = 0; i < _t.size(); i++)
        if (_t[i] != other._t[i])
            return false;
    return true;
}


bool VectorClock::operator <= (const VectorClock& other) const
{
    if (_t.size() != other._t.size())
        return false;
    for (unsigned int i = 0; i < _t.size(); i++)
        if (_t[i] > other._t[i])
            return false;
    return true;
}


bool VectorClock::operator < (const VectorClock& other) const
{
    bool iguales = true;

    if (_t.size() != other._t.size())
        return false;
    for (unsigned int i = 0; i < _t.size(); i++)
        if (_t[i] > other._t[i])
            return false;
        else if (_t[i] < other._t[i])
            iguales = false;
    return ! iguales;
}

VectorClock& VectorClock::operator += (const VectorClock& other)
{
    if (_t.size() != other._t.size())
        return *this;
    for (unsigned int i = 0; i < _t.size(); i++)
        _t[i] = std::max(_t[i], other._t[i]);
    return *this;
}


unsigned int& VectorClock::operator [] (int i)
{
    return _t[i];
}


std::ostream& operator << (std::ostream& out, const VectorClock& r)
{
    std::stringstream s;

    s << "[";
    for (unsigned int i = 0; i < r._t.size(); i++) {
        s << r._t[i];
        if (i+1 < r._t.size())
            s << ",";
    }
    s << "]";
    out << s.str();
    return out;
}


void VectorClock::resize(unsigned int newSize)
{
    _t.resize(newSize, 0);
}
